(in-package #:cl-user)

(defpackage tbnl-araneida
  (:use #:cl)
  (:export #:start #:stop))

(in-package tbnl-araneida)

; Used this to start and stop tbnl-araneida
;     (tbnl-araneida:start)
;     (tbnl-araneida:stop)

;; This is based on the example.lisp distributed with Araneida 0.9

(eval-when (:compile-toplevel :load-toplevel :execute)
  ;; This is for using Araneida behind a reverse-proxying 
  ;; Apache (or other HTTP proxy) server.  You may need to alter the 
  ;; configuration for your local setup
  (pushnew :araneida-uses-proxy *features*)

  ;; if you have a threaded SBCL, and want to, you can use the (new,
  ;; whizzy) thread-based server instead of the (older, better tested)
  ;; SERVE-EVENT thing
  #+araneida-threads (pushnew :araneida-uses-threads *features*))

#+sbcl
(defun my-fqdn ()
  (sb-bsd-sockets:host-ent-name 
   (sb-bsd-sockets:get-host-by-name (machine-instance))))

(defvar *demo-port* 8000)
(defvar *fwd-port* 9024)
(defvar *demo-url*
  (araneida:make-url :scheme "http" :host "localhost" :port *demo-port*))

#-araneida-uses-proxy
(defvar *listener*
  (make-instance #+araneida-uses-threads 'araneida:threaded-http-listener
		 #-araneida-uses-threads 'araneida:serve-event-http-listener
		 :port *demo-port*))

#+araneida-uses-proxy
(defvar *listener*
  (let ((fwd-url (araneida:copy-url *demo-url*)))
    (setf (araneida:url-port fwd-url) *fwd-port*)
    (make-instance #+araneida-uses-threads 'araneida:threaded-reverse-proxy-listener
		   #-araneida-uses-threads 'araneida:serve-event-reverse-proxy-listener
		   :translations
		   `((,(araneida:urlstring *demo-url*) ,(araneida:urlstring fwd-url)))
		   :address #(0 0 0 0)
		   :port (araneida:url-port fwd-url))))

(defun show-page ()
  "<html><head><title>HELLO</title></head><body><h1>HELLO from TBNL and Araneida</h1><image src='/tbnl-araneida/image/image.jpg' /><p><a href='/tbnl-araneida/form'>fill in a form</a></p></body></html>")

(defun show-form ()
  (let* ((name (tbnl:post-parameter "name"))
         (comment (tbnl:post-parameter "comment")))
    (format nil "<html><head><title>FORM</title></head><body><h1>FILL THIS IN</h1>
<form name='a-form' action='/tbnl-araneida/form' method='post'>
  name: <input id='name' name='name' type='text' size='30' value='~a' /><br />
  comment: <input id='comment' name='comment' type='text' size='30' value='~a' /><br />
  <input id='ok' name='ok' type='submit' value='OK' />
</form>
</body></html>" (if name name "") (if comment comment ""))))

(defun setup-tbnl-araneida ()
  (setf tbnl:*show-lisp-errors-p* t)
  (setf tbnl:*show-lisp-backtraces-p* t)
  (setf tbnl::*rewrite-for-session-urls* nil)

  (let ((tbnl-araneida (make-instance 'tbnl:tbnl-araneida-handler
                                      :port *demo-port* :fwd-port *fwd-port*)))
    (araneida:install-handler (araneida:http-listener-handler *listener*)
                              tbnl-araneida 
                              (araneida:urlstring (araneida:merge-url *demo-url* "/tbnl-araneida"))
                              nil))
  (setq tbnl:*dispatch-table*
      (nconc
       (list (tbnl:create-static-file-dispatcher-and-handler
              "/tbnl-araneida/image/image.jpg"
              (make-pathname :name "fz" :type "jpg" :version nil
                             :defaults (load-time-value *load-pathname*))
              "image/jpeg"))
       
       (mapcar (lambda (args)
                 (apply #'tbnl:create-prefix-dispatcher args))
               '(("/tbnl-araneida/form" show-form)
                 ("/tbnl-araneida" show-page)))
       (list #'tbnl:default-dispatcher))))

(defun stop ()
  (tbnl:stop-tbnl)
  (when (ARANEIDA::http-listener-threads *listener*)
    (araneida:stop-listening *listener*)))

(defun start ()
  (tbnl:reset-sessions)
  (stop)
  (setup-tbnl-araneida)
  (tbnl:start-tbnl)
  (araneida:start-listening *listener*))
